#include "common.h"

static int debug_level = 0, thrnum = 2;

void debug_msg( char *title ) {
   char msg[ 80 ] = "";
   struct timeval tv;
   if( debug_level < 1 ) return;
   gettimeofday( &tv, NULL );
   put_timeval( &tv, msg );
   sprintf( msg + strlen( msg ), " : %s\t: pid=%lu, tid=%lu",
            title, (long)getpid(), pthread_self() );
   printf( "%s\n", msg );
}

static int active = 0, numb = 30;

void *test( void *in ) {   // потоковая функция
   int i = (int)in, j;
   char msg[ 80 ];
   if( debug_level > 0 ) {
      sprintf( msg, "pthread #%d started", i );
      debug_msg( msg );
   }
   for( j = 0; j < numb / thrnum; j++ ) {
      slice( active );
      sched_yield();
   }
   if( debug_level > 0 ) {
      sprintf( msg, "pthread #%d finished", i );
      debug_msg( msg );
   }
   return NULL;
}

void final( struct timeval* tms, struct timeval* tmf ) {
   tmf->tv_sec -= tms->tv_sec;
   tmf->tv_usec -= tms->tv_usec;
   if( tmf->tv_usec < 0 ) {
      tmf->tv_sec--; tmf->tv_usec += 1000000L;
   };
   {  char msg[ 80 ];
      if( tmf->tv_sec < 60 )
         sprintf( msg, "%02lu sec. %03lu msec.", tmf->tv_sec, tmf->tv_usec / 1000 );
      else
         sprintf( msg, "%lu min. %02lu sec. %03lu msec.",
                  ( tmf->tv_sec / 60 ), ( tmf->tv_sec % 60 ), tmf->tv_usec / 1000 );
      printf( "threads number = %d, repeat in thread %d times : full time was %s\n",
              thrnum, numb / thrnum, msg );
   }
}

int main( int argc, char **argv, char **envp ) {
   int c, i; 
   char sopt[] = "n:t:a:v";
   struct timeval tms, tmf;
   pthread_t *tid;
   while( -1 != ( c = getopt( argc, argv, sopt ) ) )
      switch( c ) {
         case 'n':        // число повторений на поток
            numb = atoi( optarg );
            break;
         case 't':        // число потоков
            thrnum = atoi( optarg );
            break;
         case 'a':        // процент активности (0...100)
            active = atoi( optarg );
            break;
         case 'v':
            debug_level++;
            break;
         default :
            printf( "option must be: %s\n", sopt );
            return( EXIT_FAILURE );
      }
   tid = (pthread_t*)calloc( thrnum, sizeof( pthread_t ) );
   gettimeofday( &tms, NULL );
   debug_msg( "main started" );
   for( i = 0; i < thrnum; i++ ) 
      if( pthread_create( &tid[ i ], NULL, test, (void*)i ) != 0 )
         perror( "pthread_create" ), exit( EXIT_FAILURE );
   for( i = 0; i < thrnum; i++ )
      pthread_join( tid[ i ], NULL );
   gettimeofday( &tmf, NULL );
   debug_msg( "main finished" );
   free( tid );
   final( &tms, &tmf );
   return( EXIT_SUCCESS );
}
